
import sys
from PyQt5.QtWidgets import QWidget, QLabel, QTextEdit, QPushButton, QGridLayout, QHBoxLayout, QVBoxLayout, QApplication
from PyQt5.QtGui import QFont
from PyQt5 import QtCore

class TicTacToe():
    def __init__(self):
        self.Reset()
        
    def Reset(self):
        self.A = [['b','b','b'],['b','b','b'],['b','b','b']]
        self.CurrentPlayer = 'X' # Player 'X' always starts the game
        self.N = 0

    #        
    # Check if the cell (i,j) is already taken by a player
    def CellAlreadyTaken(self,i,j):  
         if self.A[i][j] == 'b':
            return(False)
         else:
            return(True)

    #
    # Player movement is recorded in Matrix A
    def Play(self,ID,i,j):
            self.A[i][j] = ID

    #
    # Change which player will play next            
    def ChangePlayer(self):
        if self.CurrentPlayer == 'X':
            self.CurrentPlayer = 'O'
        else:
            self.CurrentPlayer = 'X'

    #
    # Check if Player ID='X','O' has won
    def CheckPlayerWins(self,ID):
      A = self.A
      # check lines, columns and diagonals
      if ((A[0][0] == ID) and (A[0][1] == ID) and (A[0][2] == ID)) or \
         ((A[1][0] == ID) and (A[1][1] == ID) and (A[1][2] == ID)) or \
         ((A[2][0] == ID) and (A[2][1] == ID) and (A[2][2] == ID)) or \
         ((A[0][0] == ID) and (A[1][0] == ID) and (A[2][0] == ID)) or \
         ((A[0][1] == ID) and (A[1][1] == ID) and (A[2][1] == ID)) or \
         ((A[0][2] == ID) and (A[1][2] == ID) and (A[2][2] == ID)) or \
         ((A[0][0] == ID) and (A[1][1] == ID) and (A[2][2] == ID)) or \
         ((A[0][2] == ID) and (A[1][1] == ID) and (A[2][0] == ID)):
             return(True)
      else:
             return(False)

    #
    # Check if matrix A still have blank spaces, i.e.,
    # 'b'
    def CheckBlank(self):
        A = self.A
        if (A[0][0] == 'b') or (A[0][1] == 'b') or (A[0][2] == 'b') or \
           (A[1][0] == 'b') or (A[1][1] == 'b') or (A[1][2] == 'b') or \
           (A[2][0] == 'b') or (A[2][1] == 'b') or (A[2][2] == 'b'):
            return(True)
        else:
            return(False)
        
    #
    # Check if the game ended
    def GameEnds(self):
        if self.CheckPlayerWins('X'):
           return(True)
        elif self.CheckPlayerWins('O'):
           return(True)
        elif not self.CheckBlank():
           return(True)
        else:
           return(False)
    #         
    # Checks the state of the game 
    def GameState(self):
       if self.CheckPlayerWins('X'):
           return(True,'Player X')
       elif self.CheckPlayerWins('O'):
           return(True,'Player O')
       elif not self.CheckBlank():
           return(True,'The_Old_lady')
       else:
           return(False,'Still_Game')
    
    # Imprime a mensagem no console        
    def SendMessage(self,message):
        print(message)

    # Imprime a matrix A[][] no console
    def PrintA(self):
        A = self.A
        print('Jogada =',self.N)
        for i in range(3):
                print(A[i][0],' ',A[i][1],' ',A[i][2])
        print()
    

class Janela(QWidget):   
    def __init__(self):
        super(Janela, self).__init__()
        
        # Variaveis de status do Jogo        
        self.Reset = True    # You can reset it at any time
        self.GameOn = False  # sinalizes that game is not finished        
        # The state of the game: 
        self.Jogo = TicTacToe()
        # Operacoes de criacao da interface grafica
        self.initUI()        

    # Here you must declare your algorithm for Player 'X'
    def PlayerXMove(self):        
        self.WriteMessage('Player X choice is -> (i,j)')

    #
    # The control of the game !
    #
    def Controle(self,i,j,pressedbutton):
        if not self.Jogo.CellAlreadyTaken(i,j):
            # Player ID selects cell (i,j)
            # Change matrix A
            self.Jogo.Play(self.Jogo.CurrentPlayer,i,j)
            self.Jogo.N = self.Jogo.N + 1
            #
            #
            self.SetButtonGrid(pressedbutton,self.Jogo.CurrentPlayer)           
             
            if not self.Jogo.GameEnds():        # The game ended ?
                  self.Jogo.ChangePlayer()      # Change CurrentPlayer to the Opposite
                  if self.Jogo.CurrentPlayer == 'X':
                     # change label colors
                     self.lbl1.setStyleSheet('background: green')
                     self.lbl2.setStyleSheet('background: transparent')
                     # Player X chooses which cell(i,j) to play
                     self.PlayerXMove()
                  else: # self.Jogo.CurrentPlayer = 'O'
                     self.lbl1.setStyleSheet('background: transparent')
                     self.lbl2.setStyleSheet('background: green')                      
            else: # The game ended
                endstate = self.Jogo.GameState()
                thewinner = endstate[1]              # String of the winner
                self.WriteMessage('The Winner is -> '+ thewinner)
                self.GameOn = False
        else: # the cell (i,j) is taken ! choose another one empty !
            self.WriteMessage('Cell is already taken !')
        self.Jogo.PrintA()            
    
    #
    # Actions for the reset button
    def press_button_reset(self):
        print('Reset')
        # Reset the Grafical Interface
        self.ResetAllGridButtons()
        self.lbl1.setStyleSheet('background: transparent')
        self.lbl2.setStyleSheet('background: transparent')        
        self.textEdit.clear()
        # Reset the actual game, creates a new game
        self.Jogo = TicTacToe()
        self.Reset = True
        self.GameOn = False
        print('chegou aqui')
    #
    # Action for the start button
    def press_button_start(self):        
        if self.Reset == True:
             # set flags
             self.Reset = False
             self.GameOn = True 
             # Player X must play
             self.lbl1.setStyleSheet('background: green')
             self.textEdit.append(str('Start - Player X'))
             self.PlayerXMove()           
        else: # Reset == False    
             self.WriteMessage('Please Reset First !')                      
        
    # Action for the buttons of the grid layout
    def press_button(self,i,j,pressed):
        # which button of the grid has been pressed
        print('i =',i,' j =',j)
        if self.GameOn == True:
           # Lets transfer to other function
           self.Controle(i,j,pressed)
        else:
           self.WriteMessage('Game is over ! Please Reset !')
           
    # Write messages in the text area
    def WriteMessage(self,message):
        self.textEdit.append(message)

    # Set parameters of widgets: button size, font type and size
    def SetPar(self, x):  
        x.setFixedSize(80,80)
        x.setFont(QFont('SansSerif', 30))

    # Set what text goes on button: ' ', 'X' or 'O'
    def SetButtonGrid(self, button, ID):
        button.setText(ID)
        
    # Make all buttons of the grid blank text
    def ResetAllGridButtons(self):
        for i in range(9):
            self.listb[i].setText(' ')

    def initUI(self):
        StartButton = QPushButton('START')
        StartButton.setFixedSize(80,40)        
        StartButton.setFont(QFont('SansSerif', 16))
        StartButton.clicked.connect((lambda: self.press_button_start()))
        
        ResetButton = QPushButton('RESET')
        ResetButton.setFixedSize(80,40)
        ResetButton.setFont(QFont('SansSerif', 16))
        ResetButton.clicked.connect((lambda: self.press_button_reset()))
        box1 = QHBoxLayout()
        box1.addWidget(StartButton)
        box1.addWidget(ResetButton)
        box1.setAlignment(QtCore.Qt.AlignCenter)
        
        self.lbl1 = QLabel('Player X')
        self.lbl1.setFont(QFont('SansSerif',16))
        self.lbl2 = QLabel('Player O')
        self.lbl2.setFont(QFont('SansSerif',16))        
        hbox1 = QHBoxLayout()
        hbox1.addWidget(self.lbl1)
        hbox1.addWidget(self.lbl2)
        hbox1.setAlignment(QtCore.Qt.AlignCenter)
        
        grid = QGridLayout()
        button1 = QPushButton('')
        self.SetPar(button1)
        button1.clicked.connect(lambda: self.press_button(0,0,button1))
        button2 = QPushButton('')
        self.SetPar(button2)
        button2.clicked.connect(lambda: self.press_button(0,1,button2))        
        button3 = QPushButton('')
        self.SetPar(button3)
        button3.clicked.connect(lambda: self.press_button(0,2,button3))
        button4 = QPushButton('')
        self.SetPar(button4)
        button4.clicked.connect(lambda: self.press_button(1,0,button4))
        button5 = QPushButton('')
        self.SetPar(button5)
        button5.clicked.connect(lambda: self.press_button(1,1,button5))
        button6 = QPushButton('')
        self.SetPar(button6)
        button6.clicked.connect(lambda: self.press_button(1,2,button6))
        button7 = QPushButton('')
        self.SetPar(button7)
        button7.clicked.connect(lambda: self.press_button(2,0,button7))
        button8 = QPushButton('')
        self.SetPar(button8)
        button8.clicked.connect(lambda: self.press_button(2,1,button8))
        button9 = QPushButton('')
        self.SetPar(button9)
        button9.clicked.connect(lambda: self.press_button(2,2,button9))

        self.listb =[button1,button2,button3,button4,button5,button6,button7,button8,button9]        
        
        grid.addWidget(button1, 0,0)
        grid.addWidget(button2, 0,1)
        grid.addWidget(button3, 0,2)
        grid.addWidget(button4, 1,0)
        grid.addWidget(button5, 1,1)
        grid.addWidget(button6, 1,2)
        grid.addWidget(button7, 2,0)
        grid.addWidget(button8, 2,1)        
        grid.addWidget(button9, 2,2)
        grid.setAlignment(QtCore.Qt.AlignCenter)

        textArea = QHBoxLayout()
        self.textEdit = QTextEdit()
        textArea.addWidget(self.textEdit)
       
        mainbox = QVBoxLayout()
        mainbox.addLayout(box1,10)
        mainbox.addLayout(hbox1,10)
        mainbox.addLayout(grid,60)
        mainbox.addLayout(textArea,20)
 
        self.setLayout(mainbox)
        
        self.setGeometry(300, 300, 450, 450)
        self.setWindowTitle('O Jogo Da Velha')    
        self.show()
     
def main():    
    app = QApplication(sys.argv)
    ex = Janela()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()